-- GLOBALS: unpack, VoidStorageFrame

--[[//////////////////////////////////////////////////

    GLO-LO

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    local _
    local select = select
    local table = table
    local tostring = tostring
    local next = next
    local tonumber = tonumber
    local pairs = pairs
    local ipairs = ipairs
    local math = math
    local string = string
    local type = type
    local strsplit = strsplit
    local strmatch = strmatch
    local strfind = strfind

    local GetTime = GetTime
    local GetItemInfo = GetItemInfo
    local hooksecurefunc = hooksecurefunc
    local GetAddOnInfo = GetAddOnInfo
    local GetRealmName = GetRealmName
    local UnitName = UnitName
    local UnitFactionGroup = UnitFactionGroup
    local GameTooltip = GameTooltip
    local GetCurrentGuildBankTab = GetCurrentGuildBankTab
    local PlaySoundFile = PlaySoundFile
    local GetVoidItemHyperlinkString = GetVoidItemHyperlinkString
    local GetContainerItemInfo = GetContainerItemInfo
    local GetInventoryItemLink = GetInventoryItemLink
    local GetItemQualityColor = GetItemQualityColor
    local GetContainerNumSlots = GetContainerNumSlots
    local GetInventorySlotInfo = GetInventorySlotInfo
    local GetContainerItemLink = GetContainerItemLink
    local ContainerIDToInventoryID = ContainerIDToInventoryID
    local BankButtonIDToInvSlotID = BankButtonIDToInvSlotID
    local IsArtifactPowerItem = IsArtifactPowerItem
    local GetItemCount = GetItemCount
    local GetItemIcon = GetItemIcon
    local GetCoinIcon = GetCoinIcon
    local FauxScrollFrame_GetOffset = FauxScrollFrame_GetOffset
    local FauxScrollFrame_Update = FauxScrollFrame_Update
    local StaticPopup_Show = StaticPopup_Show
    local IsShiftKeyDown = IsShiftKeyDown
    local CloseDropDownMenus = CloseDropDownMenus
    local IsVoidStorageReady = IsVoidStorageReady
    local GetVoidItemInfo = GetVoidItemInfo
    local GetGuildBankItemInfo = GetGuildBankItemInfo
    local GetGuildBankItemLink = GetGuildBankItemLink
    local GetNumAuctionItems = GetNumAuctionItems
    local GetAuctionItemInfo = GetAuctionItemInfo
    local GetAuctionItemLink = GetAuctionItemLink
    local GetAuctionItemTimeLeft = GetAuctionItemTimeLeft
    local GetInboxNumItems = GetInboxNumItems
    local GetInboxHeaderInfo = GetInboxHeaderInfo
    local GetInboxItem = GetInboxItem
    local GetInboxItemLink = GetInboxItemLink
    local GetNumGuildBankTabs = GetNumGuildBankTabs
    local IsInGuild = IsInGuild
    local GetGuildInfo = GetGuildInfo
    local GetGuildBankTabInfo = GetGuildBankTabInfo

    local LibStub = LibStub

    local B, L, T = unpack(select(2, ...))
    local Icon = LibStub("LibDBIcon-1.0")
    local LibDD = LibStub:GetLibrary("LibUIDropDownMenu-4.0")

    T.TOOLTIPLABEL = {
        ['bank'] = L[10013],
        ['reagent'] = L[10014],
        ['void'] = L[10016],
        ['guild'] = L[10017],
        ['player'] = L[10015],
        ['equipped'] = L[10012],
        ['auctions'] = L[9001],
        ['mail'] = L[9000]
    }


--[[//////////////////////////////////////////////////

    MAIN FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]
    
    function B.CloseSpecialWindows()
        T.F.mainframe.lastbank = nil
    end

    function B.Initialize()
        local enabled, addons
        
        hooksecurefunc("ContainerFrameItemButton_OnModifiedClick", B.ContainerFrameItemButton_OnModifiedClick)
        hooksecurefunc("ChatFrame_OnHyperlinkShow", B.ChatFrame_OnHyperlinkShow)
        hooksecurefunc("ChatEdit_InsertLink", B.InsertLink)
        hooksecurefunc("SetItemButtonQuality", B.ButtonListener) -- Magic workaround multiple event triggering
        hooksecurefunc("CloseSpecialWindows", B.CloseSpecialWindows)

        addons = {"Adibags", "ElvUI", "Bagnon"} -- Addons that modify frame names
        for x = 1, #addons do
            _, _, _, enabled = GetAddOnInfo(addons[x])
            if enabled then
                T.BW.forceBankScan = true
                break
            end
        end

        local LibExtraTip = LibStub("LibExtraTip-1")
        LibExtraTip:AddCallback({type = "item", callback = B.TooltipListener})
        LibExtraTip:RegisterTooltip(GameTooltip)
        LibExtraTip:RegisterTooltip(BattlePetTooltip)

        local player = UnitName('player')
        local server = GetRealmName()
        local faction = UnitFactionGroup('player')

        B.Load(player, server, faction)

        local BanksWatchLDB = LibStub("LibDataBroker-1.1"):NewDataObject("BanksWatchMinimap", {
            type = "data source",
            text = "Bank's Watch",
            icon = T.icon,
            iconR = T.CLASS.r,
            iconG = T.CLASS.g,
            iconB = T.CLASS.b,
            OnClick = function(self, button)
                if button == "LeftButton" then
                    B.Main("toggle")
                elseif button == "RightButton" then
                    if not InterfaceOptionsFrame:IsShown() then
                        InterfaceOptionsFrame_Show() -- Open options
                    end
                    InterfaceOptionsFrame_OpenToCategory(B.C['panel']) -- Bank's Watch
                end
            end,
            OnTooltipShow = function()
                GameTooltip:SetText("Bank's Watch")
                GameTooltip:AddLine(" ")
                GameTooltip:AddLine("|cffffffff"..L[10066])
                GameTooltip:AddLine("|cffffffff"..L[10067])
            end
        })

        Icon:Register("BanksWatchMinimap", BanksWatchLDB, BWsavedtable['minimapicon'])
        if BWsavedtable['minimapicon']['hide'] then
            Icon:Hide("BanksWatchMinimap")
        else
            Icon:Show("BanksWatchMinimap")
        end
    end

    function B.Load(player, server, faction)

        if (player == nil) or (player == L[9002]) or (player == L[9003]) then
            return
        end

        if BWsavedtable and BWsavedtable[server] and BWsavedtable[server][player.."."..string.sub(faction, 1, 1)] then
            B.load(server, player, faction)
        else
            B.print(T.MSGC.EVENT.."New character detected")
            B.template(server, player, faction)
        end
    end

    function B.Main(input)

        if input then input = string.lower(input) end
        local input, arg1, val1, val2 = strsplit(" ", input)

        if input == "toggle" then
            if T.F.toggleframe:IsVisible() then
                T.F.toggleframe:Hide()
            else
                T.F.toggleframe:Show()
            end

        elseif input == "minimap" then
            BWsavedtable['minimapicon']['hide'] = not BWsavedtable['minimapicon']['hide']
            if BWsavedtable['minimapicon']['hide'] then
                Icon:Hide("BanksWatchMinimap")
            else
                Icon:Show("BanksWatchMinimap")
            end

        elseif input == "debug" then
            BWdebug = not BWdebug
            B.print(T.MSGC.INFO.."Debug:|r "..T.MSGC.BOOL..tostring(BWdebug), 1)

        elseif input == "item" then
            T.F.itemframe:Show()

        elseif input == "money" then

            if arg1 then
                if arg1 == "scale" then
                -- verification to make sure nothing silly is happening
                    if not tonumber(val1) then B.print(T.MSGC.ERROR..L[10055], 1); return end

                    val1 = math.max(val1, 75)
                    val1 = math.min(val1, 200)

                    T.CONFIG['scalemoneytracker'] = tonumber(val1)
                    B.print(T.MSGC.INFO.."Scaling:|r "..tostring(T.CONFIG['scalemoneytracker'] / 100), 1)
                    B.M.Frame.Scale(val1)
                    B.Main("money test")

                elseif arg1 == "test" then
                    B.M.Trigger(tonumber(val1 or 100000))

                elseif arg1 == "toggle" then
                    T.CONFIG['enablemoneytracker'] = not T.CONFIG['enablemoneytracker']
                    B.print(T.MSGC.INFO.."Small money tracker:|r "..tostring(T.CONFIG['enablemoneytracker']), 1)
                end
            end
        end
    end

    function B.ButtonListener(button, quality, link)
        -- Workaround to using events which may trigger more than once.
        if T.BW.loaded then
            if B.Guild:IsSlot(button) then
                B.Guild:ScanSlot(GetCurrentGuildBankTab(), button:GetID(), button, link)
            
            elseif B.Character:IsBag(button) then
                B.Character:ScanSlot(button:GetParent():GetID(), button:GetID())

            elseif B.Bank:IsBag(button) then
                B.Bank:ScanSlot(button:GetParent():GetID(), button:GetID())
            
            elseif B.Reagent:IsSlot(button) then
                B.Reagent:ScanSlot(button:GetID())

            elseif B.Bank:IsGenericSlot(button) then
                B.Bank:ScanGenericSlot(button:GetID())

            elseif B.Void:IsSlot(button) then
                B.Void:ScanSlot(tonumber(VoidStorageFrame.page), button:GetID(), button)
            end
        end
    end

    local newstring
    local order = {"player", "equipped", "bank", "reagent", "void", "guild", "mail", "auctions"}
    function B.TooltipListener(tooltip, itemlink, quantity)
        -- Minimal approach
        newstring = newstring or B.newtable()

        local result, itemid, bank, bag, p, k, v, total, total_server, guild, empty_line, server_line, class_c
        local player = UnitName('player')
        local faction = string.sub(UnitFactionGroup('player'), 1, 1)
        local server = GetRealmName()
        local alt, altfaction, altserver

        local r, g, b = T.TOOLTIP.r, T.TOOLTIP.g, T.TOOLTIP.b --0.74, 0.46, 1.00, nice purple...
        local contrast = B.hex(T.TOOLTIPAC.r, T.TOOLTIPAC.g, T.TOOLTIPAC.b)

        local total_players = 0
        local found_alts = 0
        local ignored = {
            [6948] = true, -- Hearthstone
            [110560] = true, -- Garrison Hearthstone
            [140192] = true -- Dalaran Hearthstone
        }

        if itemlink then
            itemid = tonumber(B.GetItemIdFromLink(itemlink))
            if ignored[itemid] then return end

            table.wipe(newstring)
            total = 0
            total_server = 0
            empty_line = false

            -- Main player
            for x = 1, #order do
                bank = order[x]
                bag = T.BW.tooltipcache[server][player.."."..faction][bank]

                if bag then
                    result = bag[itemid]
                    if result then
                        newstring[#newstring + 1] = contrast..T.TOOLTIPLABEL[bank].."|r: |cffffffff"..result.."|r"
                        if bank == "guild" or bank == "mail" then
                            total_server = total_server + result
                        elseif bank ~= "auctions" then
                            total = total + result
                        end
                        total_players = total_players + result
                    end
                end
            end

            if next(newstring) then
                tooltip:AddLine(" ")
                found_alts = found_alts + 1
                empty_line = true
                server_line = total_server > 0 and "/|cffffd100"..total_server + total.."|r " or ""
                tooltip:AddDoubleLine("|c"..T.RAID_CLASS_COLORS[T['class']].colorStr..player.."|r"..contrast.."|r:|cffffffff "..total.." |r"..server_line..contrast.."|r", table.concat(newstring, ", "), r, g, b, r, g, b)
            end

            -- Alts from same server
            table.wipe(newstring)
            for p, bank in pairs(T.BW.tooltipcache[server]) do
                if p ~= player.."."..faction then
                    alt, altfaction = p:match"([^.]*).(.*)"
                    if altfaction == faction or BWsavedtable['enablesearchallfactions'] then
                        for x = 1, #order do
                            bank = order[x]
                            bag = T.BW.tooltipcache[server][p][bank]

                            if bag then
                                result = bag[itemid]
                                if result then
                                    newstring[#newstring + 1] = contrast..T.TOOLTIPLABEL[bank].."|r: |cffffffff"..result.."|r"
                                    total_players = total_players + result
                                end
                            end
                        end
                        if next(newstring) then
                            if not empty_line then tooltip:AddLine(" "); empty_line = true end
                            found_alts = found_alts + 1
                            class_c = BWsavedtable[server][p]['player']['class'] and T.RAID_CLASS_COLORS[BWsavedtable[server][p]['player']['class']].colorStr or "ffffffff"
                            tooltip:AddDoubleLine("|c"..class_c..alt.."|r"..(BWsavedtable['enablesearchallfactions'] and " "..altfaction or ""), table.concat(newstring, ", "), r, g, b, r, g, b)
                        end
                        table.wipe(newstring)
                    end
                end
            end

            -- Alts from other server
            if BWsavedtable['enablesearchallservers'] then
                table.wipe(newstring)
                for p, bank in pairs(T.BW.tooltipcache) do
                    if p ~= server then
                        empty_line = false
                        for k, v in pairs(bank) do
                            alt, altfaction = k:match"([^.]*).(.*)"
                            if BWsavedtable['enablesearchallfactions'] or altfaction == faction then
                                for x = 1, #order do
                                    bank = order[x]
                                    bag = T.BW.tooltipcache[p][k][bank]

                                    if bag then
                                        result = bag[itemid]
                                        if result then
                                            newstring[#newstring + 1] = contrast..T.TOOLTIPLABEL[bank].."|r: |cffffffff"..result.."|r"
                                            total_players = total_players + result
                                        end
                                    end
                                end
                                if next(newstring) then
                                    if not empty_line then tooltip:AddLine(" "); tooltip:AddLine(p); empty_line = true end
                                    found_alts = found_alts + 1
                                    class_c = BWsavedtable[p][k]['player']['class'] and T.RAID_CLASS_COLORS[BWsavedtable[p][k]['player']['class']].colorStr or "ffffffff"
                                    tooltip:AddDoubleLine("|c"..class_c..alt.."|r"..(BWsavedtable['enablesearchallfactions'] and " "..altfaction or ""), table.concat(newstring, ", "), r, g, b, r, g, b)
                                end
                                table.wipe(newstring)
                            end
                        end
                    end
                end
            end

            if found_alts > 1 then
                tooltip:AddLine(" ")
                tooltip:AddDoubleLine("", "|cffffd100Total:|r |cffffffff"..total_players.."|r", r, g, b, r, g, b)
            end
        end
    end

    function B.ContainerFrameItemButton_OnModifiedClick(self)

        if IsShiftKeyDown() then
            local bag, item = self:GetParent():GetID(), self:GetID()
            local link = GetContainerItemLink(bag, item)

            if T.F.mainframe.searchbar:HasFocus() and T.F.mainframe.searchbar:GetText() ~= link then

                T.F.mainframe.searchbar:SetText(GetItemInfo(link))
            end
        end
    end

    function B.ChatFrame_OnHyperlinkShow(self, link, text, button)

        if IsShiftKeyDown() and link then
            if string.find(link, 'achievement:*') or string.find(link,'spell:*') then
                return false
            end

            if T.F.mainframe.searchbar:HasFocus() and T.F.mainframe.searchbar:GetText() ~= text then
                T.F.mainframe.searchbar:SetText(GetItemInfo(link))
            end
        end
    end

    function B.InsertLink(text)

        if T.F.mainframe.searchbar:HasFocus() and T.F.mainframe.searchbar:GetText() ~= text then
            --T.F.mainframe.searchbar:SetText(string.find(text, "^|c%x+|H(.+)|h%[.*%]"))
            T.F.mainframe.searchbar:SetText(text)
        end
    end

    function B.UpdateBanks()

        B.Bank:Update()
        B.Reagent:Update()
        B.Void:Update()
        B.Guild:Update()
        B.Character:Update()
        B.Equipment:Update()
        B.Auctions:Update()
        B.Mail:Update()

        local text = T.F.mainframe.searchbar:GetText()
        if text and text ~= "" then
            T.F.mainframe.searchbar:SetText("")
            T.F.mainframe.searchbar:SetText(text)
        else
            B.Search:Search(false)
        end
    end


--[[//////////////////////////////////////////////////

    SCAN FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    function B.Scan(slot, link, count)

        if link then
            slot = slot or B.newtable()
            slot['link'] = link
            slot['count'] = count and count > 1 and count or nil
        else
            slot = B.deltable(slot)
        end

        return slot
    end

    function B.BankOfflineScan(bag)
        -- Item count is always updated. Check the difference for the bag.
        local slot, count, total
        local bank = T['bank']

        total = 0
        for x = 1, bank[bag]['slots'] do
            slot = bank[bag][x]
            if slot and next(slot) then
                count = (GetItemCount(slot['link'], true) or 0) - (GetItemInfo(slot['link']) or 0)
                B.print(slot['link'])
                B.print(count)
                --[[if count then -- item just changed
                    B.print(slot['link'])
                    B.print(count)
                    B.print(slot['count'])
                end]]
            end
        end
    end

--[[//////////////////////////////////////////////////

    UPDATE FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]
    

    function B.RefreshPlayerBag(player, bag)
        local slots

        if not player[bag] then
            player[bag] = B.newtable()
            slots = GetContainerNumSlots(bag)
            player[bag]['slots'] = slots

            if bag > 0 then
                local inventoryID = ContainerIDToInventoryID(bag)
                local baglink = GetInventoryItemLink("player", inventoryID)
                player[bag]['link'] = baglink
            end
        end
    end

    function B.UpdateBagTab(tab, data)
        local quality, r, g, b

        if data.link then
            quality = B.GetQualityFromLink(data.link)
            r, g, b = GetItemQualityColor(quality)

            tab:SetBackdropBorderColor(r, g, b, 1)
            tab.link = data.link
            tab.quality = quality
            tab:SetNormalTexture(GetItemIcon(data.link))
            tab:Show()
        else
            tab:Hide()
        end
    end

    function B.UpdatePage(page, maxslots, data, optstart)
        local x, item
        optstart = optstart or 0

        for x = 1, maxslots do
            B.UpdateSlot(page.buttons[x + optstart], data and data[x] or nil)
        end
    end

    function B.UpdateSlot(button, data)
        local item

        if button then
            B.ResetButton(button)

            if data then
                item = B.GetItem(data.link)

                if next(item) and (button.link ~= data.link or button.count ~= (data.count or 1)) then
                    B.UpdateButton(button, item, data)
                end
            end
        end
    end

    local item
    function B.GetItem(itemlink)

        local itemid, name, link, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice
        item = item or B.newtable()

        if T.BW.loaded then
            
            if strmatch(itemlink, "|Hbattlepet:") then
                name, link, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice = B.GetPetInfo(itemlink)
            elseif strmatch(itemlink, "|Hkeystone:") then
                name, link, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice = B.GetKeystoneInfo(itemlink)
            else
                name, link, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice = GetItemInfo(itemlink)
            end

            -- Reuse the same table to avoid generating garbage
            item[1] = name
            item[2] = link
            item[3] = quality
            item[4] = iLevel
            item[5] = reqLevel
            item[6] = class
            item[7] = subclass
            item[8] = maxStack
            item[9] = equipSlot
            item[10] = texture
            item[11] = vendorPrice


            itemid = B.GetItemIdFromLink(itemlink)

            if not next(item) and itemid then -- Item is not cached yet
                T.BW.lookuprequest = true
                T.BW.lookup[tonumber(itemid)] = true
            end
        end

        return item
    end

    function B.UpdateButton(button, item, data)
        local r, g, b, ilvl

        r, g, b = GetItemQualityColor(item[3])

        button.name = item[1]
        button.count = data.count or 1
        button.quality = item[3]
        button.link = item[2]
        button.class = item[6]
        button.subclass = item[7]
        button.equipslot = item[9]
        button.sell = item[11] and item[11] > 0 and item[11] or nil

        button.icon:SetTexture(item[10])
        button.icon:SetTexCoord(0.1, 0.9, 0.1, 0.9)
        
        if (data.count or 1) > 1 then button.text:SetText(data.count) end
        if item[3] > 1 then 
            if button.bg then
                button.bg:SetVertexColor(r, g, b, 1)
            else
                button:SetBackdropBorderColor(r, g, b, 1)
            end
         end

        if item[6] == L[9006] then -- QUEST
            if button.bg then
                button.bg:SetVertexColor(1, 1, 0, 1)
            else
                button:SetBackdropBorderColor(1, 1, 0, 1)
            end
            button.quest = true

        elseif (item[6] == L[9004] or item[6] == L[9005]) and (button.quality and button.quality > 1) then -- ILVL for green quality or higher
            ilvl = T.BW.ilvlloaded and B.DetectILVL(item[2]) or item[4] > 1 and item[4] or nil

            button.ilvl:SetText(ilvl)
            button.ilvl:SetTextColor(r, g, b, 1)
        end
    end

    function B.ResetButton(button)

        button.icon:SetTexture("")

        if button.bg then
            button.bg:SetVertexColor(T.TRANSPARENCY_SLOT.r, T.TRANSPARENCY_SLOT.g, T.TRANSPARENCY_SLOT.b, T.TRANSPARENCY_SLOT.a)
        else
            button:SetBackdropBorderColor(T.BORDER_SLOT.r, T.BORDER_SLOT.g, T.BORDER_SLOT.b, T.BORDER_SLOT.a)
        end
        button.link = nil
        button.quality = nil
        button.quest = nil
        button.name = nil
        button.count = nil
        button.text:SetText("")
        button.ilvl:SetText("")
    end

--[[//////////////////////////////////////////////////

    TOOLTIP CACHE FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    function B.CacheItems(cache, itemid, count)
        if not itemid then return end -- keystone don't have itemids

        itemid = tonumber(itemid)

        if not cache[itemid] then
            cache[itemid] = 0
        end

        cache[itemid] = cache[itemid] + count
    end

    function B._CacheItems(cache, location, bank)
        local bags, item, b

        if not bank then return end

        if not cache[location] then
            cache[location] = B.newtable()
        end

        table.wipe(cache[location])

        if location == "equipped" then
            for _, item in pairs(bank) do
                if type(item) == "table" then

                    B.CacheItems(cache[location], B.GetItemIdFromLink(item['link']), item['count'] or 1)
                end
            end
        else
            for b, bags in pairs(bank) do
                if (location == "player" and b ~= "class" and b > -1) or location ~= "player" then
                    if type(bags) == "table" then
                        for _, item in pairs(bags) do
                            if type(item) == "table" then

                                B.CacheItems(cache[location], B.GetItemIdFromLink(item['link']), item['count'] or 1)
                            end
                        end
                    end
                end
            end
        end
    end

    function B.CacheMainItems(bank)
        -- Cache everything from active player into a workable table by itemid - count only.
        local banks, selected
        local player = UnitName('player')
        local faction = string.sub(UnitFactionGroup('player'), 1, 1)
        local server = GetRealmName()
        local cachename = player.."."..faction

        T.BW.tooltipcache[server] = T.BW.tooltipcache[server] or B.newtable()
        T.BW.tooltipcache[server][cachename] = T.BW.tooltipcache[server][cachename] or B.newtable()

        if not bank then -- Cache everything
            banks = {"bank", "reagent", "void", "guild", "player", "equipped", "auctions", "mail"}
            for bank = 1, #banks do
                selected = banks[bank] == "equipped" and T['player'][-1] or T[banks[bank]]
                B._CacheItems(T.BW.tooltipcache[server][cachename], banks[bank], selected)
            end
        else
            selected = bank == "equipped" and T['player'][-1] or T[bank]
            B._CacheItems(T.BW.tooltipcache[server][cachename], bank, selected)
        end
    end

    function B.CacheAltItems() -- Run only once
        local player, faction, p, b, banks, bank, selected

        for k, v in pairs(BWsavedtable) do
            if not T.OPTIONS[k] and type(v) == "table" then
                for p, b in pairs(v) do
                    B.CacheAlt(k, p, b)
                end
            end
        end
    end

    function B.CacheAlt(server, playerfaction, t)
        local player, faction, cachename, selected

        player, faction = playerfaction:match"([^.]*).(.*)"

        if player == UnitName('player') and faction == string.sub(UnitFactionGroup('player'), 1, 1) then return end

        if BWsavedtable['enablesearchallfactions'] or faction == string.sub(UnitFactionGroup('player'), 1, 1) then
            cachename = player.."."..faction

            T.BW.tooltipcache[server] = T.BW.tooltipcache[server] or B.newtable()
            T.BW.tooltipcache[server][cachename] = B.newtable()

            local banks = {"bank", "reagent", "void", "guild", "player", "equipped", "auctions", "mail"}
            for bank = 1, #banks do
                selected = banks[bank] == "equipped" and t['player'][-1] or t[banks[bank]]
                B._CacheItems(T.BW.tooltipcache[server][cachename], banks[bank], selected)
            end
        end
    end

