-- GLOBALS: unpack, select, LibStub, ITEM_SEARCHBAR_LIST
-- GLOBALS: VoidStorageFrame, ReagentBankFrame, MerchantFrame

--[[//////////////////////////////////////////////////

    GLO-LO

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    local _G = _G
    local math = math
    local tostring = tostring
    local tonumber = tonumber
    local string = string
    local pairs = pairs
    local format = format
    local tinsert = tinsert
    local strmatch = strmatch
    local strsplit = strsplit

    local CreateFrame = CreateFrame
    local UIParent = UIParent
    local ResetCursor = ResetCursor
    local SetCursor = SetCursor
    local ShowInspectCursor = ShowInspectCursor
    local GameTooltip = GameTooltip
    local GameTooltip_Hide = GameTooltip_Hide
    local GameTooltip_ShowCompareItem = GameTooltip_ShowCompareItem
    local BattlePetToolTip_Show = BattlePetToolTip_Show
    local BattlePetTooltip = BattlePetTooltip
    local CloseDropDownMenus = CloseDropDownMenus
    local ToggleDropDownMenu = ToggleDropDownMenu
    local UIDropDownMenu_CreateInfo = UIDropDownMenu_CreateInfo
    local UIDropDownMenu_SetWidth = UIDropDownMenu_SetWidth
    local UIDropDownMenu_Initialize = UIDropDownMenu_Initialize
    local UIDropDownMenu_AddButton = UIDropDownMenu_AddButton
    local UseContainerItem = UseContainerItem
    local PickupContainerItem = PickupContainerItem
    local PickupGuildBankItem = PickupGuildBankItem
    local GetItemQualityColor = GetItemQualityColor
    local SplitContainerItem = SplitContainerItem
    local SplitGuildBankItem = SplitGuildBankItem
    local ClickVoidStorageSlot = ClickVoidStorageSlot
    local GetCurrentGuildBankTab = GetCurrentGuildBankTab
    local AutoStoreGuildBankItem = AutoStoreGuildBankItem
    local GetCVarBool = GetCVarBool
    local IsShiftKeyDown = IsShiftKeyDown
    local IsModifiedClick = IsModifiedClick
    local IsControlKeyDown = IsControlKeyDown
    local HandleModifiedItemClick = HandleModifiedItemClick
    local SocketContainerItem = SocketContainerItem
    local GetMoneyString = GetMoneyString
    local ColorPickerFrame = ColorPickerFrame
    local FauxScrollFrame_OnVerticalScroll = FauxScrollFrame_OnVerticalScroll
    local IsBagOpen, OpenBag = IsBagOpen, OpenBag
    local IsArtifactPowerItem = IsArtifactPowerItem
    local SocketInventoryItem = SocketInventoryItem
    local SpellCanTargetItem = SpellCanTargetItem
    local SpellCanTargetItemID = SpellCanTargetItemID
    local DressUpItemLink = DressUpItemLink
    local SplitStack = SplitStack
    local ContainerFrameItemButton_OnClick = ContainerFrameItemButton_OnClick
    local StackSplitFrame = StackSplitFrame

    local B, L, T = unpack(select(2, ...))
    local LibDD = LibStub:GetLibrary("LibUIDropDownMenu-4.0")
    B.Base = {['Options'] = {}}


--[[//////////////////////////////////////////////////

    CREATE FRAME AND BUTTONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


    function B.Base:CreateFrame()

        local frame = CreateFrame("Frame", "BWmainframe", T.F.toggleframe, "BackdropTemplate")
        frame:SetPoint("CENTER")
        frame:SetHeight(25)
        frame:SetWidth(15 + (32 * 6) + 192) -- options + searchbar
        frame:SetFrameStrata("DIALOG")
        frame:SetMovable(true)
        frame:SetUserPlaced(true)
        frame:SetClampedToScreen(true)
        frame.layout = 0
        frame:SetScript("OnMouseDown", self.MouseDown)
        frame:SetScript("OnMouseUp", self.MouseUp)
        frame:SetScript("OnHide", self.Hide)

        B.I.Backdrop(frame)
        T.F.mainframe = frame

        self:CreateButton()
        B.Toggle:CreateButton()
        B.Bank:CreateButton()
        B.Reagent:CreateButton()
        B.Character:CreateButton()
        B.Void:CreateButton()
        B.Guild:CreateButton()
    end

    function B.Base:MouseDown()
        self:StartMoving()
    end

    function B.Base:MouseUp()
        self:StopMovingOrSizing()
    end

    function B.Base:Hide()
        LibDD:CloseDropDownMenus()
    end


    function B.Base:CreateButton()
        -- Create option button which is the anchor
        local button = B.I.Main.Frame.Button(nil, nil, "BWoptionbutton")
        button:SetHighlightTexture("")
        button:SetPoint("TOPLEFT", T.F.mainframe, "TOPLEFT")
        button:SetPoint("BOTTOM", T.F.mainframe, "BOTTOM")
        button:SetBackdropColor(0.07, 0.07, 0.07, 1)
        button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
        button:RegisterForDrag("LeftButton")
        button:SetScript("OnShow", self.Show)
        button:SetScript("OnEnter", self.Enter)
        button:SetScript("OnLeave", self.Leave)
        button:SetScript("OnDragStart", self.DragStart)
        button:SetScript("OnDragStop", self.DragStop)
        button:SetScript("OnClick", self.Click)

        self.Options:Create()
        T.F.mainframe.optionbutton = button
    end

    function B.Base:Show()
        T.F.status:SetParent(T.F.mainframe.optionbutton)
        T.F.status:SetVertexColor(T.CLASS.r, T.CLASS.g, T.CLASS.b, 1)
        B.SetFrameLocation()
    end

    function B.Base:Enter()

        T.F.status:SetVertexColor(1, 1, 1, 1)
        GameTooltip:ClearLines()
        GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
        GameTooltip:SetText(T.ACTIVE_TABLE.player.." ("..string.sub(T.ACTIVE_TABLE.faction, 1, 1)..")")
        GameTooltip:AddLine(GetMoneyString(B.GetPlayerBank('currency')['money'] or 0, true))
        if not T.F.mainframe.alt then
            GameTooltip:AddLine(" ")
            GameTooltip:AddDoubleLine(L[10074], GetMoneyString(T.MONEY_GAINED, true)) --gained
            GameTooltip:AddDoubleLine(L[10073], GetMoneyString(T.MONEY_SPENT, true)) --spent
            if T.MONEY_GAINED > 0 and T.MONEY_SPENT > 0 and T.MONEY_GAINED > T.MONEY_SPENT then
                GameTooltip:AddDoubleLine(L[10075], GetMoneyString(T.MONEY_GAINED - T.MONEY_SPENT, true)) --profit
            end
        end
        GameTooltip:AddLine(" ")
        GameTooltip:AddLine(L[10005], 1, 1, 1) -- drag
        GameTooltip:AddLine(L[10006], 1, 1, 1) -- hide
        GameTooltip:AddLine(L[10007], 1, 1, 1) -- menu
        GameTooltip:Show()
    end

    function B.Base:Leave()
        T.F.status:SetVertexColor(T.CLASS.r, T.CLASS.g, T.CLASS.b, 1)
        GameTooltip:Hide()
    end

    function B.Base:DragStart()
        T.F.status:SetVertexColor(0.18, 0.64, 1, 1)
        T.F.mainframe:StartMoving()
    end

    function B.Base:DragStop()
        T.F.status:SetVertexColor(T.CLASS.r, T.CLASS.g, T.CLASS.b, 1)
        T.F.mainframe:StopMovingOrSizing()
        B.SetFrameLocation()
    end

    function B.Base:Click(button)

        if button == "RightButton" then
            T.F.mainframe.togglebutton:Show()
        else
            LibDD:ToggleDropDownMenu(1, nil, _G['BWdropDownMenu'], self:GetName(), 0, 0)
        end 
    end


    function B.Base.Options:Create()

        local menu = LibDD:Create_UIDropDownMenu("BWdropDownMenu", T.F.mainframe)
        menu.point = "TOPLEFT"
        menu.relativePoint = "BOTTOMLEFT"
        menu:Hide()

        LibDD:UIDropDownMenu_SetWidth(menu, 130, 4)
        LibDD:UIDropDownMenu_Initialize(menu, B.Base.Options.Initialize, "MENU") --The virtual
    end

    function B.Base.Options:Initialize(level, menulist)
        --drop down menues can have sub menues. The value of level determines the drop down sub menu tier
        local UIDROPDOWNMENU_MENU_VALUE = L_UIDROPDOWNMENU_MENU_VALUE
        local BWsavedtable = BWsavedtable
        local level = level or 1
        local x, k, v, class, checked, faction, player

        if level == 1 then
            local info = LibDD:UIDropDownMenu_CreateInfo()

            if BWsavedtable then
                for k, v in pairs(BWsavedtable) do
                    if not T.OPTIONS[k] then
                        B.Base.Options:CreateMenu(info, k, k, level)
                    end
                end
            end

            LibDD:UIDropDownMenu_AddSpace(level)
            if T.CONFIG then
                --- Enable search all servers
                B.Base.Options:CreateCheck(info, L[10059], "searchallservers", BWsavedtable['enablesearchallservers'], level)
                --- Enable search opposite faction
                B.Base.Options:CreateCheck(info, L[10060], "searchallfactions", BWsavedtable['enablesearchallfactions'], level)
            end

            LibDD:UIDropDownMenu_AddSpace(level)
            if T.ACTIVE_TABLE.player then
                B.Base.Options:CreateButton(info, L[10011].." "..T.ACTIVE_TABLE.player, "delete", level)
            end

            LibDD:UIDropDownMenu_AddSeparator(level)
            --- Money gain tracker
            B.Base.Options:CreateMenu(info, L[10038], "moneytracker", level)

            LibDD:UIDropDownMenu_AddSpace(level)
            if T.MONEY_GAINED > 0 or T.MONEY_SPENT > 0 then
                B.Base.Options:CreateButton(info, L[10076], "moneysessionreset", level)
            end
            B.Base.Options:CreateButton(info, L[10072], "interfaceoptions", level)

        elseif level == 2 and UIDROPDOWNMENU_MENU_VALUE == "moneytracker" then
            local info = LibDD:UIDropDownMenu_CreateInfo()
            --- Enable tracker
            B.Base.Options:CreateCheck(info, L[10039], "enablemoneytracker", T.CONFIG['enablemoneytracker'], level)
            -- Set minimum
            B.Base.Options:CreateButton(info, L[10050], "minimummoneytracker", level)
            --- Set extra money value
            --B.I.OptionsDropDown.Button(self, info, L[10051], "treasuremoneytracker", false, level)
            --- Move tracker
            B.Base.Options:CreateButton(info, L[10042], "movemoneytracker", level)
            --- Enable sounds
            LibDD:UIDropDownMenu_AddSpace(level)
            B.Base.Options:CreateCheck(info, L[10047], "enablesoundsmoneytracker", T.CONFIG['enablesoundsmoneytracker'], level)
            --- Mone
            LibDD:UIDropDownMenu_AddSeparator(level)
            B.Base.Options:CreateCheck(info, L[10045], "coinssound", T.CONFIG['moneysound'] == 1 and true or false, level)
            --- Retro coins
            B.Base.Options:CreateCheck(info, L[10046], "retrocoinssound", T.CONFIG['moneysound'] == 2 and true or false, level)
            --- Treasure 1
            --B.I.OptionsDropDown.Button(self, info, L[10048], "treasure1sound", T.CONFIG['treasuresound'] == 3 and true or false, level)
            --- Treasure 2
            --B.I.OptionsDropDown.Button(self, info, L[10049], "treasure2sound", T.CONFIG['treasuresound'] == 4 and true or false, level)

        elseif level == 2 then -- server
            local info = LibDD:UIDropDownMenu_CreateInfo()

            for k, v in pairs(BWsavedtable[UIDROPDOWNMENU_MENU_VALUE]) do
                checked = tostring(T.ACTIVE_TABLE.player.."."..string.sub(T.ACTIVE_TABLE.faction, 1, 1)) == k and true or false
                player, faction = k:match"([^.]*).(.*)"
                class = "|c"..(v['player']['class'] and T.RAID_CLASS_COLORS[v['player']['class']].colorStr or "ffffffff")
                B.Base.Options:CreateCheck(info, player.." ("..faction..")", UIDROPDOWNMENU_MENU_VALUE.."."..k, checked, level, class)
            end
        end
    end

    function B.Base.Options:CreateMenu(info, label, value, level)

        info.text = label
        info.value = value
        info.hasArrow = true
        info.notCheckable = true
        info.keepShownOnClick = true
        info.func = nil

        LibDD:UIDropDownMenu_AddButton(info, level)
    end

    function B.Base.Options:CreateButton(info, label, value, level, hex, icon)

        info.text = label
        info.hasArrow = false
        info.value = value
        info.func = B.Base.Options.Click
        info.owner = frame
        info.notCheckable = true
        info.colorCode = hex

        LibDD:UIDropDownMenu_AddButton(info, level)
    end


    function B.Base.Options:CreateCheck(info, label, value, checked, level, hex, icon)

        info.text = label
        info.hasArrow = false
        info.value = value
        info.func = B.Base.Options.Click
        info.owner = frame
        info.checked = checked
        info.notCheckable = false
        info.colorCode = hex

        LibDD:UIDropDownMenu_AddButton(info, level)
    end

    function B.Base.Options:Click()
        local server, player, faction
        local value = self.value

        if not value then return end

        if value == "delete" then
            BWsavedtable[T.ACTIVE_TABLE.server][T.ACTIVE_TABLE.player.."."..string.sub(T.ACTIVE_TABLE.faction, 1, 1)] = nil

            -- Verify if server still needs to exists.
            if not next(BWsavedtable[T.ACTIVE_TABLE.server]) then BWsavedtable[T.ACTIVE_TABLE.server] = nil end

            local player = UnitName('player')
            local server = GetRealmName()
            local faction = UnitFactionGroup('player')
            B.Load(player, server, faction)

            if B.IsCurrentPlayer() then
                B.Character:ScanBags()
                B.Equipment:Scan()
            end

            B.UpdateBanks()
            -- Cache items here
            B.CacheMainItems()
            B.CacheAltItems()

            T.F.mainframe.character.auctions:Hide()
            T.F.mainframe.character.mail:Hide()

        elseif value == "searchallservers" then
            BWsavedtable['enablesearchallservers'] = not BWsavedtable['enablesearchallservers']
            return

        elseif value == "searchallfactions" then
            BWsavedtable['enablesearchallfactions'] = not BWsavedtable['enablesearchallfactions']
            return

        elseif value == "movemoneytracker" then
            T.F.moneyframe.anchor:Show()

        elseif value == "enablesoundsmoneytracker" then
            T.CONFIG['enablesoundsmoneytracker'] = not T.CONFIG['enablesoundsmoneytracker']
            return
        elseif value == "enablemoneytracker" then
            T.CONFIG['enablemoneytracker'] = not T.CONFIG['enablemoneytracker']
            return
        elseif value == "minimummoneytracker" then StaticPopup_Show("BWsetminimum")
        elseif value == "treasuremoneytracker" then StaticPopup_Show("BWsettreasure")
        elseif value == "coinssound" then
            T.CONFIG['moneysound'] = 1
            PlaySoundFile(T.SOUNDS[T.CONFIG['moneysound']], "Master")
            return

        elseif value == "retrocoinssound" then
            T.CONFIG['moneysound'] = 2
            PlaySoundFile(T.SOUNDS[T.CONFIG['moneysound']], "Master")
            return
        elseif value == "interfaceoptions" then
            if not InterfaceOptionsFrame:IsShown() then
                InterfaceOptionsFrame_Show() -- Open options
            end
            InterfaceOptionsFrame_OpenToCategory(B.C['panel'])
        elseif value == "moneysessionreset" then
            T.MONEY_SPENT = 0
            T.MONEY_GAINED = 0
        --elseif value == "treasure1sound" then
            --T.CONFIG['treasuresound'] = 3
            --PlaySoundFile(T.SOUNDS[T.CONFIG['treasuresound']], "Master")
            --return

        --elseif value == "treasure2sound" then
            --T.CONFIG['treasuresound'] = 4
            --PlaySoundFile(T.SOUNDS[T.CONFIG['treasuresound']], "Master")
            --return
        else
            server, player, faction = value:match"([^.]*).([^.]*).(.*)"
            faction = faction == "H" and "Horde" or faction == "A" and "Alliance" or faction == "N" and "Neutral"
            B.load(server, player, faction)
            B.UpdateBanks()
        end

        LibDD:CloseDropDownMenus()
    end
