-- GLOBALS: unpack, select, LibStub, ITEM_SEARCHBAR_LIST
-- GLOBALS: VoidStorageFrame, ReagentBankFrame, MerchantFrame

--[[//////////////////////////////////////////////////

    GLO-LO

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    local _G = _G
    local math = math
    local tostring = tostring
    local tonumber = tonumber
    local string = string
    local pairs = pairs
    local format = format
    local tinsert = tinsert
    local strmatch = strmatch
    local strsplit = strsplit

    local CreateFrame = CreateFrame
    local UIParent = UIParent
    local ResetCursor = ResetCursor
    local SetCursor = SetCursor
    local ShowInspectCursor = ShowInspectCursor
    local GameTooltip = GameTooltip
    local GameTooltip_Hide = GameTooltip_Hide
    local GameTooltip_ShowCompareItem = GameTooltip_ShowCompareItem
    local BattlePetToolTip_Show = BattlePetToolTip_Show
    local BattlePetTooltip = BattlePetTooltip
    local CloseDropDownMenus = CloseDropDownMenus
    local ToggleDropDownMenu = ToggleDropDownMenu
    local UIDropDownMenu_CreateInfo = UIDropDownMenu_CreateInfo
    local UIDropDownMenu_SetWidth = UIDropDownMenu_SetWidth
    local UIDropDownMenu_Initialize = UIDropDownMenu_Initialize
    local UIDropDownMenu_AddButton = UIDropDownMenu_AddButton
    local UseContainerItem = UseContainerItem
    local PickupContainerItem = PickupContainerItem
    local PickupGuildBankItem = PickupGuildBankItem
    local GetItemQualityColor = GetItemQualityColor
    local SplitContainerItem = SplitContainerItem
    local SplitGuildBankItem = SplitGuildBankItem
    local ClickVoidStorageSlot = ClickVoidStorageSlot
    local GetCurrentGuildBankTab = GetCurrentGuildBankTab
    local AutoStoreGuildBankItem = AutoStoreGuildBankItem
    local GetCVarBool = GetCVarBool
    local IsShiftKeyDown = IsShiftKeyDown
    local IsModifiedClick = IsModifiedClick
    local IsControlKeyDown = IsControlKeyDown
    local HandleModifiedItemClick = HandleModifiedItemClick
    local SocketContainerItem = SocketContainerItem
    local GetMoneyString = GetMoneyString
    local ColorPickerFrame = ColorPickerFrame
    local FauxScrollFrame_OnVerticalScroll = FauxScrollFrame_OnVerticalScroll
    local IsBagOpen, OpenBag = IsBagOpen, OpenBag
    local IsArtifactPowerItem = IsArtifactPowerItem
    local SocketInventoryItem = SocketInventoryItem
    local SpellCanTargetItem = SpellCanTargetItem
    local SpellCanTargetItemID = SpellCanTargetItemID
    local DressUpItemLink = DressUpItemLink
    local SplitStack = SplitStack
    local ContainerFrameItemButton_OnClick = ContainerFrameItemButton_OnClick
    local StackSplitFrame = StackSplitFrame

    local B, L, T = unpack(select(2, ...))
    B.Character = {['Tooltip'] = {}, ['Bags'] = {}, ['Default'] = {}, ['Bag'] = {}}


--[[//////////////////////////////////////////////////

    CREATE FRAME AND BUTTONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


    function B.Character:CreateFrame()

        local frame = CreateFrame("Frame", "BWcharacter", T.F.mainframe, "BackdropTemplate")
        frame:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
        frame:SetFrameStrata("HIGH")
        frame:SetScript("OnShow", B.Character.Show)
        frame:SetScript("OnHide", B.Character.Hide)

        B.I.Backdrop(frame)
        T.F.mainframe.character = frame
    end

    function B.Character:Show()
        T.F.mainframe.characterbutton:LockHighlight()
    end

    function B.Character:Hide()
        T.F.mainframe.characterbutton:UnlockHighlight()
    end



    function B.Character:CreateButton()

        local button = B.I.Main.Frame.Button("|cffffffff"..L[10010].."|r")
        button:SetBackdropColor(T.CHARTAB.r, T.CHARTAB.g, T.CHARTAB.b, T.CHARTAB.a)
        button:SetScript("OnClick", B.Character.Click)

        T.F.mainframe.characterbutton = button
    end

    function B.Character:Click(...)

        local frame = self:GetParent()

        frame.bank:Hide()
        frame.reagent:Hide()
        frame.void:Hide()
        frame.guild:Hide()
        frame.search:Hide()

        if frame.character:IsVisible() then
            frame.lastbank = nil
            frame.character:Hide()
        else
            frame.lastbank = "character"
            frame.character:Show()
        end
    end



    function B.Character:CreateLayout()

        T.F.mainframe.character.tooltiptoggle = B.I.Create_FrameButton(L[10058], T.F.mainframe.character, B.Character.Tooltip)
        T.F.mainframe.character.bagstoggle = B.I.Create_FrameButton(L[10057], T.F.mainframe.character, B.Character.Bags)

        B.Character:CreateBags()
        B.Character:CreateContainers()
        B.Character:CreateButtons()
        B.Character:CreateTabs()

        B.Equipment:CreateFrame()
        B.Equipment:CreateButtons()
        B.Auctions:CreateFrame()
        B.Mail:CreateFrame()
        --B.Farm:CreateFrame()
    end


    function B.Character.Tooltip:Click(...)
        T.CONFIG['playerbagtooltiptoggle'] = not T.CONFIG['playerbagtooltiptoggle']
    end

    function B.Character.Tooltip:Enter()

        GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
        GameTooltip:SetText(L[10009])
        GameTooltip:Show()
    end

    function B.Character.Tooltip:Leave()
        GameTooltip:Hide()
    end


    function B.Character.Bags:Click(...)

        if T.F.mainframe.character.bags:IsVisible() then
            T.F.mainframe.character.bags:Hide()
            T.CONFIG['playerbagstoggle'] = false
        else
            T.F.mainframe.character.bags:Show()
            T.CONFIG['playerbagstoggle'] = true
        end
    end

    function B.Character.Bags:Enter()

        GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
        GameTooltip:SetText(L[10008])
        GameTooltip:Show()
    end

    function B.Character.Bags:Leave()
        GameTooltip:Hide()
    end


    function B.Character:CreateBags()
        -- Backpack, for some reason, setting table bags[0] throws an error when creating the tab frame for it.
        -- It seems [0] is probably used somewhere in the blizzard frame code... grrr...
        local x
        T.F.mainframe.character.bags = CreateFrame("Frame", nil, T.F.mainframe.character, "BackdropTemplate")
        
        for x = 1, T.NUM_BAG_SLOTS + 1 do
            T.F.mainframe.character.bags[x] = {}
            T.F.mainframe.character.bags[x].tab = B.I.Tab.Create(T.F.mainframe.character.bags, "BWcharacter"..x, "BackdropTemplate")
            T.F.mainframe.character.bags[x].tab.search:SetAllPoints(T.F.mainframe.character.bags[x].tab) -- since we have permanent borders
            T.F.mainframe.character.bags[x].tab.index = x - 1
            T.F.mainframe.character.bags[x].tab:SetScript("OnEnter", self.Bag.Enter)
            T.F.mainframe.character.bags[x].tab:SetScript("OnLeave", self.Bag.Leave)
            T.F.mainframe.character.bags[x].tab:SetScript("OnClick", self.Bag.Click)

            if x == 1 then --Backpack
                T.F.mainframe.character.bags[x].tab:SetNormalTexture("Interface\\Buttons\\Button-Backpack-Up")
                T.F.mainframe.character.bags[x].tab.index = 0
            end
        end
    end

    function B.Character.Bag:Click()
        if not T.F.mainframe.alt then ToggleBag(self.index) end
    end

    function B.Character.Bag:Enter()
        
        if (T.CONFIG and T.CONFIG['playerbagtooltiptoggle']) or not T.CONFIG then
            local r, g, b = GetItemQualityColor(self.quality or 1)
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
            if self.link then GameTooltip:SetHyperlink(self.link) end
            GameTooltip:SetBackdropBorderColor(r, g, b)
            GameTooltip:Show()
        end

        if #T.BW.items == 0 then
            B.Character.Bags:Search(self.index)
        end
    end

    function B.Character.Bag:Leave()

        GameTooltip:Hide()

        if #T.BW.items == 0 then
            B.Character.Bags:Search(false)
        end
    end

    function B.Character:CreateContainers()
        -- Create parent frames for buttons to communicate with blizzard api for clicking behavior.
        local x
        T.F.mainframe.character.container = CreateFrame("Frame", nil, T.F.mainframe.bank)

        for x = T.BACKPACK_CONTAINER, T.NUM_BAG_SLOTS do
            T.F.mainframe.character.container[x] = {}
            B.I.Frame.Create(T.F.mainframe.character, T.F.mainframe.character.container[x])
            T.F.mainframe.character.container[x].frame:SetID(x)
            T.F.mainframe.character.container[x].frame.name = "characterbag"
        end
    end

    function B.Character:CreateButtons()

        local x, b, button, rows
        B.I.Frame.Create(T.F.mainframe.character, T.F.mainframe.character)

        for x = T.BACKPACK_CONTAINER, T.NUM_BAG_SLOTS do
            for b = 1, T.MAX_BAG_SLOTS do
                button = B.I.Button.Create(b, T.F.mainframe.character.container[x].frame)
                button:Hide()
                button:SetScript("OnEnter", B.I.Button.Enter)
                button:SetScript("OnLeave", B.I.Button.Leave)
                button.SplitStack = SplitStack
                T.F.mainframe.character.container[x].buttons[b] = button
            end
        end
    end

    function B.Character:UpdateButtons()
        --Load the number of slots dynamically, based on the loaded character or alt database.
        local x, b, button
        local bank = B.GetPlayerBank('player')

        for b = 1, #T.F.mainframe.character.buttons do
            T.F.mainframe.character.buttons[b]:Hide()
        end
        T.F.mainframe.character.buttons = {}

        for x = T.BACKPACK_CONTAINER, T.NUM_BAG_SLOTS do
            if bank[x] then
                for b = 1, bank[x]['slots'] do
                    button = T.F.mainframe.character.container[x].buttons[b]
                    button:Show()
                    T.F.mainframe.character.buttons[#T.F.mainframe.character.buttons + 1] = button
                end
            end
        end

        rows = B.I.Grid.Update(T.F.mainframe.character)
        T.F.mainframe.character:SetHeight(10 + (T.BUTTON_HEIGHT * (rows + 1)) + (T.FRAMEWHITESPACE * rows))
        T.F.mainframe.character:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))

        if T.F.mainframe.character:GetHeight() < 300 then
            T.F.mainframe.character:SetHeight(300) -- Set minimum so our bag buttons don't dangle
        end
    end


    function B.Character:CreateTabs()

        B.Equipment:CreateTab()
        B.Auctions:CreateTab()
        B.Mail:CreateTab()
        --B.Farm:CreateTab()
    end


--[[//////////////////////////////////////////////////

    FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


    function B.Character:IsBag(button)

        local name = button:GetName()
        if name and strfind(name, "ContainerFrame%d+Item%d+") then
            local bagid = button:GetParent():GetID()
            if bagid > -1 and bagid <= T.NUM_BAG_SLOTS then
                return true
            end
        end
        return false
    end

    function B.Character:Update()

        local page, total, counter, quality, r, g, b
        local player = B.GetPlayerBank('player')

        B.Character:UpdateButtons(player)

        counter = 0
        for page = 0, #player do

            if page > 0 then -- Ignore generic bag
                B.UpdateBagTab(T.F.mainframe.character.bags[page + 1].tab, player[page])
            end

            if player[page] then

                B.UpdatePage(T.F.mainframe.character, player[page]['slots'], player[page], counter)
                counter = counter + player[page]['slots']
            end
        end
    end

    function B.Character:UpdateSlot(bag, slot)

        local page, counter, item
        local player = B.GetPlayerBank('player')

        B.Character:UpdateButtons(player)

        counter = 0
        for page = 0, #player do

            if page > 0 then -- Ignore generic bag
                B.UpdateBagTab(T.F.mainframe.character.bags[page + 1].tab, player[page])
            end

            if player[page] then
                if page == bag then
                    
                    B.ResetButton(T.F.mainframe.character.buttons[counter + slot])
                    if player[bag][slot] then

                        item = B.GetItem(player[bag][slot].link)
                        if next(item) then B.UpdateButton(T.F.mainframe.character.buttons[counter + slot], item, player[bag][slot]) end
                    end
                    B.UpdateBanks()
                end
                counter = counter + player[page]['slots']
            end
        end
    end

    function B.Character:ScanBags()

        local x, counter, count, link, bag, slot, slots
        local player = T['player']

        for bag = T.BACKPACK_CONTAINER, T.NUM_BAG_SLOTS do
            if not player[bag] then player[bag] = B.newtable() end

            slots = GetContainerNumSlots(bag)
            player[bag]['slots'] = slots

            if bag > 0 then
                local inventoryID = ContainerIDToInventoryID(bag)
                local baglink = GetInventoryItemLink("player", inventoryID)
                player[bag]['link'] = baglink
            end

            for x = 1, slots do
                _, count, _, _, _, _, link = GetContainerItemInfo(bag, x)

                player[bag][x] = B.Scan(player[bag][x], link, count)
            end
        end

        if B.IsCurrentPlayer() then self:Update() end
        B.CacheMainItems('player')
    end

    function B.Character:ScanBag(bag)

        local x, slots, count, link
        local player = T['player']

        if not player[bag] then player[bag] = B.newtable() end
            
        slots = GetContainerNumSlots(bag)
        player[bag]['slots'] = slots

        if bag > 0 then
            local inventoryID = ContainerIDToInventoryID(bag)
            local baglink = GetInventoryItemLink("player", inventoryID)
            player[bag]['link'] = baglink
        end

        for x = 1, slots do
            _, count, _, _, _, _, link = GetContainerItemInfo(bag, x)
            player[bag][x] = B.Scan(player[bag][x], link, count)
        end

        if B.IsCurrentPlayer() then self:Update() end
        B.CacheMainItems('player')
        -- special case doesn't call B.UpdateBanks() but still want to update search
        B.Search:Refresh()
    end

    function B.Character:ScanSlot(bag, slot)

        local player = T['player']
        local _, count, _, _, _, _, link = GetContainerItemInfo(bag, slot)

        if not player[bag] then
            player[bag] = B.newtable()
            local slots = GetContainerNumSlots(bag)
            player[bag]['slots'] = slots

            if bag > 0 then
                local inventoryID = ContainerIDToInventoryID(bag)
                local baglink = GetInventoryItemLink("player", inventoryID)
                player[bag]['link'] = baglink
            end
        end

        if not B.CompareSlot(player[bag][slot], link, count) then

            player[bag][slot] = B.Scan(player[bag][slot], link, count)
            if B.IsCurrentPlayer() then self:UpdateSlot(bag, slot) end
            B.CacheMainItems('player')
        end
    end


    function B.Character:Search(text)

        local x, page, counter, button, buttons, name
        local player = B.GetPlayerBank('player')

        local counter = 1
        for page = 0, #player do

            if player[page] then
                tab = page == 0 and 1 or page + 1

                if text then 
                    T.F.mainframe.character.bags[tab].tab.search:Show()
                else 
                    T.F.mainframe.character.bags[tab].tab.search:Hide() 
                end
                buttons = T.F.mainframe.character.buttons

                for x = 1, player[page]['slots'] do
                    button = buttons[counter]

                    if button.name and text then
                        if B.find(text, button) then
                            button.loc = "player."..tab
                            B.Search:Slot(button)
                            T.F.mainframe.character.bags[tab].tab.search:Hide()
                        else
                            button.search:Show()
                        end
                    else
                        button.search:Hide()
                    end
                    counter = counter + 1
                end
            end
        end
    end

    function B.Character.Bags:Search(bag)

        local player = B.GetPlayerBank('player')
        local x, page, counter, button, buttons

        local counter = 1
        for page = 0, #player do
            if player[page] then

                if not bag or page == bag then
                    for x = 1, player[page]['slots'] do
                        T.F.mainframe.character.buttons[counter].search:Hide()
                        counter = counter + 1
                    end
                else
                    for x = 1, player[page]['slots'] do
                        T.F.mainframe.character.buttons[counter].search:Show()
                        counter = counter + 1
                    end
                end
            end
        end
    end


--[[//////////////////////////////////////////////////

    POSITION

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


    function B.Character:Anchor(startpoint, endpoint, from, to)

        local lasttab

        T.F.mainframe.character:ClearAllPoints()
        T.F.mainframe.character:SetPoint(startpoint..from, T.F.mainframe, endpoint..from)

        B.I.Main.Frame.ButtonHighlight(T.F.mainframe.characterbutton, endpoint)

        T.F.mainframe.character.equipmenttab:ClearAllPoints()
        T.F.mainframe.character.equipmenttab:SetPoint("TOP"..from, T.F.mainframe.character, "TOP"..to)

        T.F.mainframe.character.auctionstab:ClearAllPoints()
        T.F.mainframe.character.auctionstab:SetPoint("TOP"..from, T.F.mainframe.character.equipmenttab, "BOTTOM"..from, 0, -4)

        T.F.mainframe.character.mailtab:ClearAllPoints()
        T.F.mainframe.character.mailtab:SetPoint("TOP"..from, T.F.mainframe.character.auctionstab, "BOTTOM"..from, 0, -4)

        --T.F.mainframe.character.farmtab:ClearAllPoints()
        --T.F.mainframe.character.farmtab:SetPoint("TOP"..from, T.F.mainframe.character.mailtab, "BOTTOM"..from, 0, -4)

        for x = 1, T.NUM_BAG_SLOTS + 1 do
            T.F.mainframe.character.bags[x].tab:ClearAllPoints()
            if x == 1 then
                T.F.mainframe.character.bags[x].tab:SetPoint("TOP"..from, T.F.mainframe.character.mailtab, "BOTTOM"..from, 0, -20)
            else
                T.F.mainframe.character.bags[x].tab:SetPoint("TOP"..from, lasttab, "BOTTOM"..from, 0, 0)
            end
            lasttab = T.F.mainframe.character.bags[x].tab
        end

        T.F.mainframe.character.tooltiptoggle:ClearAllPoints()
        T.F.mainframe.character.tooltiptoggle:SetPoint("TOP"..to, to == "RIGHT" and -10 or 10, 0)

        T.F.mainframe.character.bagstoggle:ClearAllPoints()
        T.F.mainframe.character.bagstoggle:SetPoint("TOP"..to, T.F.mainframe.character.tooltiptoggle, "TOP"..from, to == "RIGHT" and -5 or 5, 0)
    end


