-- GLOBALS: unpack, select, LibStub, ITEM_SEARCHBAR_LIST
-- GLOBALS: VoidStorageFrame, ReagentBankFrame, MerchantFrame

--[[//////////////////////////////////////////////////

    GLO-LO

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    local _G = _G
    local math = math
    local tostring = tostring
    local tonumber = tonumber
    local string = string
    local pairs = pairs
    local format = format
    local tinsert = tinsert
    local strmatch = strmatch
    local strsplit = strsplit

    local CreateFrame = CreateFrame
    local UIParent = UIParent
    local ResetCursor = ResetCursor
    local SetCursor = SetCursor
    local ShowInspectCursor = ShowInspectCursor
    local GameTooltip = GameTooltip
    local GameTooltip_Hide = GameTooltip_Hide
    local GameTooltip_ShowCompareItem = GameTooltip_ShowCompareItem
    local BattlePetToolTip_Show = BattlePetToolTip_Show
    local BattlePetTooltip = BattlePetTooltip
    local CloseDropDownMenus = CloseDropDownMenus
    local ToggleDropDownMenu = ToggleDropDownMenu
    local UIDropDownMenu_CreateInfo = UIDropDownMenu_CreateInfo
    local UIDropDownMenu_SetWidth = UIDropDownMenu_SetWidth
    local UIDropDownMenu_Initialize = UIDropDownMenu_Initialize
    local UIDropDownMenu_AddButton = UIDropDownMenu_AddButton
    local UseContainerItem = UseContainerItem
    local PickupContainerItem = PickupContainerItem
    local PickupGuildBankItem = PickupGuildBankItem
    local GetItemQualityColor = GetItemQualityColor
    local SplitContainerItem = SplitContainerItem
    local SplitGuildBankItem = SplitGuildBankItem
    local ClickVoidStorageSlot = ClickVoidStorageSlot
    local GetCurrentGuildBankTab = GetCurrentGuildBankTab
    local AutoStoreGuildBankItem = AutoStoreGuildBankItem
    local GetCVarBool = GetCVarBool
    local IsShiftKeyDown = IsShiftKeyDown
    local IsModifiedClick = IsModifiedClick
    local IsControlKeyDown = IsControlKeyDown
    local HandleModifiedItemClick = HandleModifiedItemClick
    local SocketContainerItem = SocketContainerItem
    local GetMoneyString = GetMoneyString
    local ColorPickerFrame = ColorPickerFrame
    local FauxScrollFrame_OnVerticalScroll = FauxScrollFrame_OnVerticalScroll
    local IsBagOpen, OpenBag = IsBagOpen, OpenBag
    local IsArtifactPowerItem = IsArtifactPowerItem
    local SocketInventoryItem = SocketInventoryItem
    local SpellCanTargetItem = SpellCanTargetItem
    local SpellCanTargetItemID = SpellCanTargetItemID
    local DressUpItemLink = DressUpItemLink
    local SplitStack = SplitStack
    local ContainerFrameItemButton_OnClick = ContainerFrameItemButton_OnClick
    local StackSplitFrame = StackSplitFrame

    local B, L, T = unpack(select(2, ...))
    B.Equipment = {}


--[[//////////////////////////////////////////////////

    CREATE FRAME AND BUTTONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


    function B.Equipment:CreateFrame()

        T.F.mainframe.character.equipment = CreateFrame("Frame", "BWcharacterequipment", T.F.mainframe.character, "BackdropTemplate")
        T.F.mainframe.character.equipment:SetWidth(20 + (T.BUTTON_HEIGHT * 5) + (T.FRAMEWHITESPACE * (5 - 1)))
        T.F.mainframe.character.equipment:SetFrameStrata("HIGH")
        T.F.mainframe.character.equipment:Hide()

        T.F.mainframe.character.equipment:SetScript("OnShow", self.Show)
        T.F.mainframe.character.equipment:SetScript("OnHide", self.Hide)

        B.I.Backdrop(T.F.mainframe.character.equipment)
    end

    function B.Equipment:Show()
        T.F.mainframe.character.equipmenttab:SetBackdropBorderColor(1, 1, 1, 1)
    end

    function B.Equipment:Hide()
        T.F.mainframe.character.equipmenttab:SetBackdropBorderColor(0, 0, 0, 1)
    end


    function B.Equipment:CreateTab()

        T.F.mainframe.character.equipmenttab = B.I.Tab.Create(T.F.mainframe.character, nil, "BackdropTemplate")
        T.F.mainframe.character.equipmenttab:SetNormalTexture("Interface\\PaperDollInfoFrame\\PaperDollSidebarTabs")
        T.F.mainframe.character.equipmenttab:GetNormalTexture():SetTexCoord(0.01562500, 0.53125000, 0.46875000, 0.60546875)
        T.F.mainframe.character.equipmenttab:SetBackdropColor(0.05, 0.05, 0.05, 1)
        T.F.mainframe.character.equipmenttab:HookScript("OnClick", self.Click)
        T.F.mainframe.character.equipmenttab:HookScript("OnEnter", self.Enter)
        T.F.mainframe.character.equipmenttab:SetScript("OnLeave", self.Leave)
    end

    function B.Equipment:Click(...)

        T.F.mainframe.character.auctions:Hide()
        T.F.mainframe.character.mail:Hide()
        --T.F.mainframe.farm:Hide()

        if T.F.mainframe.character.equipment:IsVisible() then
            self:SetBackdropBorderColor(0, 0, 0, 1)
            T.F.mainframe.character.equipment:Hide()
        else
            self:SetBackdropBorderColor(1, 1, 1, 1)
            T.F.mainframe.character.equipment:Show()
        end
    end

    function B.Equipment:Enter(...)

        if T.F.mainframe.layout > 2 then
            GameTooltip:SetOwner(self, "ANCHOR_LEFT", 10, -10)
        else
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
        end
        GameTooltip:SetText(L[9011])
        GameTooltip:Show()
    end

    function B.Equipment:Leave()
        GameTooltip:Hide()
    end


    function B.Equipment:CreateButtons()
        
        local x, button, lastbutton, lastrow, rows, num

        B.I.Frame.Create(T.F.mainframe.character.equipment, T.F.mainframe.character.equipment)
        rows, num = 0, 0

        for x = 1, T.EQUIPMENT_SLOTS do

            button = B.I.Button.Create(x, T.F.mainframe.character.equipment.frame)
            button:SetScript("OnEnter", B.I.Button.Enter)
            button:SetScript("OnLeave", B.I.Button.Leave)
            button:HookScript("OnEnter", function(self)
                if not self.link then
                    GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
                    GameTooltip:SetText(_G[T.INVSLOTS[self:GetID()]])
                    GameTooltip:Show()
                end
            end)
            
            if x == 1 then -- First slot
                button:SetPoint("TOPLEFT")
                rows, num = rows + 1, 1
                lastrow = button

            elseif num == 5 then -- Last column of the row
                button:SetPoint("TOP", lastrow, "BOTTOM", 0, -T.FRAMEWHITESPACE)
                rows, num = rows + 1, 1
                lastrow = button
            else
                button:SetPoint("TOPLEFT", lastbutton, "TOPRIGHT", T.FRAMEWHITESPACE, 0)
                num = num + 1
            end
            T.F.mainframe.character.equipment.buttons[x] = button
            lastbutton = button
        end

        T.F.mainframe.character.equipment:SetHeight(10 + (T.BUTTON_HEIGHT * (rows + 1)) + (T.FRAMEWHITESPACE * rows))
        T.F.mainframe.character.equipment:SetWidth(20 + (T.BUTTON_HEIGHT * 5) + (T.FRAMEWHITESPACE * (5 - 1)))
    end


--[[//////////////////////////////////////////////////

    FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


    function B.Equipment:Update()
        local player = B.GetPlayerBank('player')
        B.UpdatePage(T.F.mainframe.character.equipment, 18, player[-1])
    end

    function B.Equipment:UpdateSlot(slot)
        local player = B.GetPlayerBank('player')
        B.UpdateSlot(T.F.mainframe.character.equipment.buttons[slot], player[-1][slot])
    end


    function B.Equipment:Scan()

        local inventory, slotId, link
        local player = T['player']

        if not player[-1] then player[-1] = B.newtable() end

        -- Tabard slot is slotId 19?
        for x = 1, #T.INVSLOTS do

            slotId = GetInventorySlotInfo(T.INVSLOTS[x])
            link = GetInventoryItemLink("player", slotId)

            if slotId == 19 then slotId = 18 end

            player[-1][slotId] = B.Scan(player[-1][slotId], link)
        end

        B.CacheMainItems('equipped')
    end

    function B.Equipment:ScanSlot(slot)

        local player = T['player']
        local link = GetInventoryItemLink("player", slot)

        if slot == 19 then slot = 18 end

        if not B.CompareSlot(player[-1][slot], link, nil) then

            player[-1][slot] = B.Scan(player[-1][slot], link)
            if B.IsCurrentPlayer() then B.Equipment:UpdateSlot(slot) end
            B.CacheMainItems('equipped')
        end
    end


    function B.Equipment:Search(text)

        local x, button, buttons, name
        local player = B.GetPlayerBank('player')

        if not player[-1] then return end

        buttons = T.F.mainframe.character.equipment.buttons
        for x = 1, T.EQUIPMENT_SLOTS do
            button = buttons[x]

            if button.name and text then
                if B.find(text, button) then
                    button.loc = "equip.1"
                    B.Search:Slot(button)
                else
                    button.search:Show()
                end
            else
                button.search:Hide()
            end

        end
    end


--[[//////////////////////////////////////////////////

    POSITION

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


    function B.Equipment:Anchor(startpoint, endpoint, from, to)

        T.F.mainframe.character.equipment:ClearAllPoints()
        T.F.mainframe.character.equipment:SetPoint("TOP"..from, T.F.mainframe.character.equipmenttab, "TOP"..to)
    end
