-- GLOBALS: unpack, select, LibStub, ITEM_SEARCHBAR_LIST
-- GLOBALS: VoidStorageFrame, ReagentBankFrame, MerchantFrame

--[[//////////////////////////////////////////////////

    GLO-LO

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    local _G = _G
    local math = math
    local tostring = tostring
    local tonumber = tonumber
    local string = string
    local pairs = pairs
    local format = format
    local tinsert = tinsert
    local strmatch = strmatch
    local strsplit = strsplit

    local CreateFrame = CreateFrame
    local UIParent = UIParent
    local ResetCursor = ResetCursor
    local SetCursor = SetCursor
    local ShowInspectCursor = ShowInspectCursor
    local GameTooltip = GameTooltip
    local GameTooltip_Hide = GameTooltip_Hide
    local GameTooltip_ShowCompareItem = GameTooltip_ShowCompareItem
    local BattlePetToolTip_Show = BattlePetToolTip_Show
    local BattlePetTooltip = BattlePetTooltip
    local CloseDropDownMenus = CloseDropDownMenus
    local ToggleDropDownMenu = ToggleDropDownMenu
    local UIDropDownMenu_CreateInfo = UIDropDownMenu_CreateInfo
    local UIDropDownMenu_SetWidth = UIDropDownMenu_SetWidth
    local UIDropDownMenu_Initialize = UIDropDownMenu_Initialize
    local UIDropDownMenu_AddButton = UIDropDownMenu_AddButton
    local UseContainerItem = UseContainerItem
    local PickupContainerItem = PickupContainerItem
    local PickupGuildBankItem = PickupGuildBankItem
    local GetItemQualityColor = GetItemQualityColor
    local SplitContainerItem = SplitContainerItem
    local SplitGuildBankItem = SplitGuildBankItem
    local ClickVoidStorageSlot = ClickVoidStorageSlot
    local GetCurrentGuildBankTab = GetCurrentGuildBankTab
    local AutoStoreGuildBankItem = AutoStoreGuildBankItem
    local GetCVarBool = GetCVarBool
    local IsShiftKeyDown = IsShiftKeyDown
    local IsModifiedClick = IsModifiedClick
    local IsControlKeyDown = IsControlKeyDown
    local HandleModifiedItemClick = HandleModifiedItemClick
    local SocketContainerItem = SocketContainerItem
    local GetMoneyString = GetMoneyString
    local ColorPickerFrame = ColorPickerFrame
    local FauxScrollFrame_OnVerticalScroll = FauxScrollFrame_OnVerticalScroll
    local IsBagOpen, OpenBag = IsBagOpen, OpenBag
    local IsArtifactPowerItem = IsArtifactPowerItem
    local SocketInventoryItem = SocketInventoryItem
    local SpellCanTargetItem = SpellCanTargetItem
    local SpellCanTargetItemID = SpellCanTargetItemID
    local DressUpItemLink = DressUpItemLink
    local SplitStack = SplitStack
    local ContainerFrameItemButton_OnClick = ContainerFrameItemButton_OnClick
    local StackSplitFrame = StackSplitFrame

    local B, L, T = unpack(select(2, ...))
    B.Guild = {['Tabs'] = {}, ['Default'] = {}}


--[[//////////////////////////////////////////////////

    CREATE FRAME AND BUTTONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


    function B.Guild:CreateFrame()

        local frame = CreateFrame("Frame", "BWguild", T.F.mainframe, "BackdropTemplate")
        frame:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
        frame:SetFrameStrata("HIGH")
        frame.lasttab = 1

        frame:SetScript("OnShow", B.Guild.Show)
        frame:SetScript("OnHide", B.Guild.Hide)

        B.I.Backdrop(frame)
        T.F.mainframe.guild = frame
    end

    function B.Guild:Show()

        T.F.mainframe.guildbutton:LockHighlight()
        T.F.mainframe.guild[T.F.mainframe.guild.lasttab].tab:Click()
    end

    function B.Guild:Hide()
        T.F.mainframe.guildbutton:UnlockHighlight()
    end


    function B.Guild:CreateButton()
        
        local button = B.I.Main.Frame.Button("|cffffffff"..L[10004].."|r")
        button:SetBackdropColor(T.GUILDTAB.r, T.GUILDTAB.g, T.GUILDTAB.b, T.GUILDTAB.a)
        button:SetScript("OnClick", self.Click)
        button:HookScript("OnEnter", self.Enter)
        button:HookScript("OnLeave", self.Leave)

        T.F.mainframe.guildbutton = button
    end

    function B.Guild:Click(...)

        local frame = self:GetParent()

        if not T.ISGUILDOPEN then
            frame.character:Hide()
        elseif T.ISGUILDOPEN and frame.character:IsVisible() then
            -- move bags
            B.AnchorBags(frame.guild)
        end

        frame.bank:Hide()
        frame.reagent:Hide()
        frame.void:Hide()
        frame.search:Hide()

        if frame.guild:IsVisible() then
            frame.lastbank = nil
            frame.guild:Hide()
        else
            frame.lastbank = "guild"
            frame.guild:Show()
        end
    end

    function B.Guild:Enter()

        local name = B.GetPlayerBank('guild')['name']

        if name then
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
            GameTooltip:SetText(name)
            GameTooltip:Show()
        end
    end

    function B.Guild:Leave()
        GameTooltip:Hide()
    end



    function B.Guild:CreateLayout()

        B.Guild.Tabs:Create()
        B.Guild:CreateButtons()
    end


    function B.Guild.Tabs:Create()

        local x, lasttab

        for x = 1, T.MAX_GUILDBANK_TABS do
            T.F.mainframe.guild[x] = {}
            T.F.mainframe.guild[x].tab = B.I.Tab.Create(T.F.mainframe.guild, nil, "BackdropTemplate")
            T.F.mainframe.guild[x].tab.index = x
            T.F.mainframe.guild[x].tab:SetScript("OnClick", B.Guild.Tabs.Click)
            T.F.mainframe.guild[x].tab:SetScript("OnEnter", B.Guild.Tabs.Enter)
            T.F.mainframe.guild[x].tab:SetScript("OnLeave", B.Guild.Tabs.Leave)

            if x == 1 then
                T.F.mainframe.guild[x].tab:SetPoint("TOPLEFT", T.F.mainframe.guild, "TOPRIGHT")
                T.F.mainframe.guild[x].tab:SetBackdropBorderColor(1, 1, 1, 1)
            else
                T.F.mainframe.guild[x].tab:SetPoint("TOPLEFT", lasttab, "BOTTOMLEFT", 0, -4)
            end

            lasttab = T.F.mainframe.guild[x].tab
        end
    end

    function B.Guild.Tabs:Click(...)

        local parent = self:GetParent()

        parent[parent.lasttab].frame:Hide()
        parent[parent.lasttab].tab:SetBackdropBorderColor(0, 0, 0, 1)

        self:SetBackdropBorderColor(1, 1, 1, 1)
        parent[self.index].frame:Show()
        parent.lasttab = self.index

        if T.ISGUILDOPEN and GuildBankFrame then
            _G['GuildBankTab'..self.index..'Button']:Click()
        end
    end

    function B.Guild.Tabs:Enter()

        if T.F.mainframe.layout > 2 then
            GameTooltip:SetOwner(self, "ANCHOR_LEFT", 10, -10)
        else
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
        end

        if self.name then GameTooltip:SetText(self.name) end
        GameTooltip:Show()
    end

    function B.Guild.Tabs:Leave()
        GameTooltip:Hide()
    end


    function B.Guild:CreateButtons()
        
        local x, b, button, rows

        for x = 1, T.MAX_GUILDBANK_TABS do
            B.I.Frame.Create(T.F.mainframe.guild, T.F.mainframe.guild[x], x)
            T.F.mainframe.guild[x].frame:SetID(x)

            for b = 1, T.MAX_GUILDBANK_SLOTS_PER_TAB do
                button = B.I.Button.Create(b, T.F.mainframe.guild[x].frame)
                button:SetScript("OnEnter", B.I.Button.Enter)
                button:SetScript("OnLeave", B.I.Button.Leave)
                button.SplitStack = B.Guild.SplitStack
                T.F.mainframe.guild[x].buttons[b] = button
            end

            rows = B.I.Grid.Update(T.F.mainframe.guild[x])
        end

        T.F.mainframe.guild:SetHeight(10 + (T.BUTTON_HEIGHT * (rows + 1)) + (T.FRAMEWHITESPACE * rows))
        T.F.mainframe.guild:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
    end

    function B.Guild.SplitStack(button, split)
        SplitGuildBankItem(button:GetParent():GetID(), button:GetID(), split)
    end

--[[//////////////////////////////////////////////////

    FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


    function B.Guild:IsSlot(button)

        local name = button:GetName()
        if name and strfind(name, "GuildBankColumn%d+Button%d+") then
            return true
        end
        return false
    end

    function B.Guild:Update()

        local page, guild = _, B.GetPlayerBank('guild')

        for page = 1, T.MAX_GUILDBANK_TABS do
            if guild[page] then

                if guild[page].icon then
                    T.F.mainframe.guild[page].tab:SetNormalTexture(guild[page].icon)
                    T.F.mainframe.guild[page].tab.name = guild[page].name
                    T.F.mainframe.guild[page].tab:Show()
                else
                    T.F.mainframe.guild[page].tab:Hide()
                end

                if guild[page].access then -- Validate user has access to view tab
                    T.F.mainframe.guild[page].tab:Enable()
                    T.F.mainframe.guild[page].tab:GetNormalTexture():SetDesaturated(false)
                    B.UpdatePage(T.F.mainframe.guild[page], T.MAX_GUILDBANK_SLOTS_PER_TAB, guild[page])
                else
                    T.F.mainframe.guild[page].tab:Disable()
                    T.F.mainframe.guild[page].tab:GetNormalTexture():SetDesaturated(true)
                end
            else
                T.F.mainframe.guild[page].tab:Hide()
                B.UpdatePage(T.F.mainframe.guild[page], T.MAX_GUILDBANK_SLOTS_PER_TAB, nil)
            end
         end
    end

    function B.Guild:UpdateSlot(page, slot)

        local guild = B.GetPlayerBank('guild')
        B.UpdateSlot(T.F.mainframe.guild[page].buttons[slot], guild[page][slot])       
    end

    function B.Guild.Tabs:Update()

        local x, name, icon, isViewable
        local guild = T['guild']
        local tabs = GetNumGuildBankTabs()

        if IsInGuild() then guild['name'] = GetGuildInfo('player') end

        for x = 1, T.MAX_GUILDBANK_TABS do
            if x <= tabs then
                if not guild[x] then guild[x] = B.newtable() end
                name, icon, isViewable = GetGuildBankTabInfo(x)
                guild[x]['icon'] = icon
                guild[x]['name'] = name
                guild[x]['access'] = isViewable

                T.F.mainframe.guild[x].tab:SetNormalTexture(icon)
                T.F.mainframe.guild[x].tab.name = name
                T.F.mainframe.guild[x].tab:Enable()
                T.F.mainframe.guild[x].tab:GetNormalTexture():SetDesaturated(false)
                T.F.mainframe.guild[x].tab:Show()
            else
                guild[x] = B.deltable(guild[x])
                T.F.mainframe.guild[x].tab:Disable()
                T.F.mainframe.guild[x].tab:GetNormalTexture():SetDesaturated(true)
                T.F.mainframe.guild[x].tab:Hide()
            end
        end
    end

    function B.Guild:Scan()
        -- This is used to force scan the entire bank in the event the user is using a bag addon that
        -- modifies the blizzard frame and makes it impossible to scan.
        local x, count
        local guild = T['guild']
        local page = GetCurrentGuildBankTab()

        B.Guild.Tabs:Update()

        if guild[page].access then
            for x = 1, T.MAX_GUILDBANK_SLOTS_PER_TAB do

                _, count = GetGuildBankItemInfo(page, x)
                if count > 0 then
                    guild[page][x] = guild[page][x] or B.newtable()
                    guild[page][x]['link'] = GetGuildBankItemLink(page, x)
                    guild[page][x]['count'] = count > 1 and count or nil
                else
                    guild[page][x] = B.deltable(guild[page][x])
                end
            end
        end

        if B.IsCurrentPlayer() then self:Update() end
        B.CacheMainItems('guild')
    end

    function B.Guild:ScanSlot(page, slot, button, link)

        local guild = T['guild']

        if button.count > 0 then
            if strfind(link, "item:82800:") then -- Caged pet
                link = B.GetGuildPetLink(page, slot)
            end
        end

        if not B.CompareSlot(guild[page][slot], link, button.count) then

            guild[page][slot] = B.Scan(guild[page][slot], link, button.count)
            if B.IsCurrentPlayer() then self:UpdateSlot(page, slot) end
            B.CacheMainItems('guild')
        end
    end

    function B.Guild:Search(text)

        local x, page, button, buttons
        local guild = B.GetPlayerBank('guild')

        for page = 1, T.MAX_GUILDBANK_TABS do

            if text then T.F.mainframe.guild[page].tab.search:Show()
            else T.F.mainframe.guild[page].tab.search:Hide() end

            if guild[page] and guild[page].access then
                buttons = T.F.mainframe.guild[page].buttons

                for x = 1, T.MAX_GUILDBANK_SLOTS_PER_TAB do
                    button = buttons[x]

                    if button.name and text then
                        if B.find(text, button) then
                            button.loc = "guild."..page
                            B.Search:Slot(button)
                            T.F.mainframe.guild[page].tab.search:Hide()
                        else
                            button.search:Show()
                        end
                    else
                        button.search:Hide()
                    end
                end
            end
        end
    end

    function B.GuildBankItemButton_OnClick(self, button)
        -- Based on blizzard -- incase it gets overwritten by other addons...
        if ( HandleModifiedItemClick(GetGuildBankItemLink(GetCurrentGuildBankTab(), self:GetID())) ) then
            return;
        end
        if ( IsModifiedClick("SPLITSTACK") ) then
            if ( not CursorHasItem() ) then
                local texture, count, locked = GetGuildBankItemInfo(GetCurrentGuildBankTab(), self:GetID());
                if ( not locked and count and count > 1) then
                    StackSplitFrame:OpenStackSplitFrame(count, self, "BOTTOMLEFT", "TOPLEFT");
                end
            end
            return;
        end
        local type, money = GetCursorInfo();
        if ( type == "money" ) then
            DepositGuildBankMoney(money);
            ClearCursor();
        elseif ( type == "guildbankmoney" ) then
            DropCursorMoney();
            ClearCursor();
        else
            if ( button == "RightButton" ) then
                AutoStoreGuildBankItem(GetCurrentGuildBankTab(), self:GetID());
            else
                PickupGuildBankItem(GetCurrentGuildBankTab(), self:GetID());
            end
        end
    end


--[[//////////////////////////////////////////////////

    POSITION

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


    function B.Guild:Anchor(startpoint, endpoint, from, to)
        local lasttab

        T.F.mainframe.guild:ClearAllPoints()
        T.F.mainframe.guild:SetPoint(startpoint..from, T.F.mainframe, endpoint..from)

        B.I.Main.Frame.ButtonHighlight(T.F.mainframe.guildbutton, endpoint)

        for x = 1, T.MAX_GUILDBANK_TABS do
            T.F.mainframe.guild[x].tab:ClearAllPoints()
            if x == 1 then
                T.F.mainframe.guild[x].tab:SetPoint("TOP"..from, T.F.mainframe.guild, "TOP"..to)
            else
                T.F.mainframe.guild[x].tab:SetPoint("TOP"..from, lasttab, "BOTTOM"..from, 0, -4)
            end
            lasttab = T.F.mainframe.guild[x].tab
        end
    end
