-- GLOBALS: unpack, select, LibStub, ITEM_SEARCHBAR_LIST
-- GLOBALS: VoidStorageFrame, ReagentBankFrame, MerchantFrame

--[[//////////////////////////////////////////////////

    GLO-LO

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    local _G = _G
    local math = math
    local tostring = tostring
    local tonumber = tonumber
    local string = string
    local pairs = pairs
    local format = format
    local tinsert = tinsert
    local strmatch = strmatch
    local strsplit = strsplit

    local CreateFrame = CreateFrame
    local UIParent = UIParent
    local ResetCursor = ResetCursor
    local SetCursor = SetCursor
    local ShowInspectCursor = ShowInspectCursor
    local GameTooltip = GameTooltip
    local GameTooltip_Hide = GameTooltip_Hide
    local GameTooltip_ShowCompareItem = GameTooltip_ShowCompareItem
    local BattlePetToolTip_Show = BattlePetToolTip_Show
    local BattlePetTooltip = BattlePetTooltip
    local CloseDropDownMenus = CloseDropDownMenus
    local ToggleDropDownMenu = ToggleDropDownMenu
    local UIDropDownMenu_CreateInfo = UIDropDownMenu_CreateInfo
    local UIDropDownMenu_SetWidth = UIDropDownMenu_SetWidth
    local UIDropDownMenu_Initialize = UIDropDownMenu_Initialize
    local UIDropDownMenu_AddButton = UIDropDownMenu_AddButton
    local UseContainerItem = UseContainerItem
    local PickupContainerItem = PickupContainerItem
    local PickupGuildBankItem = PickupGuildBankItem
    local GetItemQualityColor = GetItemQualityColor
    local SplitContainerItem = SplitContainerItem
    local SplitGuildBankItem = SplitGuildBankItem
    local ClickVoidStorageSlot = ClickVoidStorageSlot
    local GetCurrentGuildBankTab = GetCurrentGuildBankTab
    local AutoStoreGuildBankItem = AutoStoreGuildBankItem
    local GetCVarBool = GetCVarBool
    local IsShiftKeyDown = IsShiftKeyDown
    local IsModifiedClick = IsModifiedClick
    local IsControlKeyDown = IsControlKeyDown
    local HandleModifiedItemClick = HandleModifiedItemClick
    local SocketContainerItem = SocketContainerItem
    local GetMoneyString = GetMoneyString
    local ColorPickerFrame = ColorPickerFrame
    local FauxScrollFrame_OnVerticalScroll = FauxScrollFrame_OnVerticalScroll
    local IsBagOpen, OpenBag = IsBagOpen, OpenBag
    local IsArtifactPowerItem = IsArtifactPowerItem
    local SocketInventoryItem = SocketInventoryItem
    local SpellCanTargetItem = SpellCanTargetItem
    local SpellCanTargetItemID = SpellCanTargetItemID
    local DressUpItemLink = DressUpItemLink
    local SplitStack = SplitStack
    local ContainerFrameItemButton_OnClick = ContainerFrameItemButton_OnClick
    local StackSplitFrame = StackSplitFrame

    local B, L, T = unpack(select(2, ...))
    B.Reagent = {}


--[[//////////////////////////////////////////////////

    CREATE FRAME AND BUTTONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


    function B.Reagent:CreateFrame()

        local frame = CreateFrame("Frame", "BWreagent", T.F.mainframe, "BackdropTemplate")
        frame:SetFrameStrata("HIGH")
        frame:SetScript("OnShow", B.Reagent.Show)
        frame:SetScript("OnHide", B.Reagent.Hide)

        B.I.Backdrop(frame)
        T.F.mainframe.reagent = frame
    end

    function B.Reagent:Show()
        T.F.mainframe.reagentbutton:LockHighlight()
    end

    function B.Reagent:Hide()
        T.F.mainframe.reagentbutton:UnlockHighlight()
    end


    function B.Reagent:CreateButton()

        local button = B.I.Main.Frame.Button("|cffffffff"..L[10002].."|r")
        button:SetBackdropColor(T.REAGENTTAB.r, T.REAGENTTAB.g, T.REAGENTTAB.b, T.REAGENTTAB.a)
        button:SetScript("OnClick", B.Reagent.Click)

        T.F.mainframe.reagentbutton = button
    end

    function B.Reagent:Click(...)

        local frame = self:GetParent()

        if not T.ISBANKOPEN then
            frame.character:Hide()
        elseif T.ISBANKOPEN and frame.character:IsVisible() then
            -- move bags
            B.AnchorBags(frame.bank)
        end

        frame.bank:Hide()
        frame.void:Hide()
        frame.guild:Hide()
        frame.search:Hide()

        if frame.reagent:IsVisible() then
            frame.lastbank = nil
            frame.reagent:Hide()
        else
            frame.lastbank = "reagent"
            frame.reagent:Show()
        end
    end


    function B.Reagent:CreateLayout()
        B.Reagent:CreateButtons()
    end

    function B.Reagent:CreateButtons()

        local x, b, button, rows

        for x = 1, T.REAGENT_PAGES do
            T.F.mainframe.reagent[x] = {}
            B.I.Frame.Create(T.F.mainframe.reagent, T.F.mainframe.reagent[x], x)
            T.F.mainframe.reagent[x].frame:SetID(T.REAGENTBANK_CONTAINER)

            for b = 1, T.NUM_REAGENT_SLOTS do
                button = B.I.Button.Create(b, T.F.mainframe.reagent[x].frame)
                button:SetScript("OnEnter", B.I.Button.Enter)
                button:SetScript("OnLeave", B.I.Button.Leave)
                button.SplitStack = SplitStack
                T.F.mainframe.reagent[x].buttons[b] = button
            end

            rows = B.I.Grid.Update(T.F.mainframe.reagent[x])
        end

        T.F.mainframe.reagent:SetHeight(10 + (T.BUTTON_HEIGHT * (rows + 1)) + (T.FRAMEWHITESPACE * rows))
        T.F.mainframe.reagent:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
    end


--[[//////////////////////////////////////////////////

    FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


    function B.Reagent:IsSlot(button)
        
        local name = button:GetName()
        if name and strfind(name, "ReagentBankFrameItem%d+") then
            return true
        end
        return false
    end

    function B.Reagent:ScanSlot(slot)

        local reagent = T['reagent']
        local _, count, _, _, _, _, link = GetContainerItemInfo(T.REAGENTBANK_CONTAINER, slot)

        if not reagent[1] then reagent[1] = B.newtable() end

        if not B.CompareSlot(reagent[1][slot], link, count) then

            reagent[1][slot] = B.Scan(reagent[1][slot], link, count)
            if B.IsCurrentPlayer() then self:UpdateSlot(slot) end
            B.CacheMainItems('reagent')
        end
    end

    function B.Reagent:Update()

        local page, reagent = _, B.GetPlayerBank('reagent')

        for page = 1, T.REAGENT_PAGES do
            B.UpdatePage(T.F.mainframe.reagent[page], T.NUM_REAGENT_SLOTS, reagent[page])
        end
    end

    function B.Reagent:UpdateSlot(slot)

        local reagent = B.GetPlayerBank('reagent')
        B.UpdateSlot(T.F.mainframe.reagent[1].buttons[slot], reagent[1][slot])
    end

    function B.Reagent:Search(text)

        local x, page, button, buttons
        local reagent = B.GetPlayerBank('reagent')

        for page = 1, T.REAGENT_PAGES do
            buttons = T.F.mainframe.reagent[page].buttons

            for x = 1, T.NUM_REAGENT_SLOTS do
                button = buttons[x]

                if button.name and text then
                    if B.find(text, button) then
                        button.loc = "reagent."..page
                        B.Search:Slot(button)
                    else
                        button.search:Show()
                    end
                else
                    button.search:Hide()
                end
            end
        end
    end


--[[//////////////////////////////////////////////////

    POSITION

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


    function B.Reagent:Anchor(startpoint, endpoint, from, to)

        T.F.mainframe.reagent:ClearAllPoints()
        T.F.mainframe.reagent:SetPoint(startpoint..from, T.F.mainframe, endpoint..from)

        B.I.Main.Frame.ButtonHighlight(T.F.mainframe.reagentbutton, endpoint)
    end

